#ifndef CYGONCE_PLF_IO_H
#define CYGONCE_PLF_IO_H

//=============================================================================
//
//      plf_io.h
//
//      Platform specific IO support
//
//=============================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    hmt, jskov, nickg
// Contributors: hmt, jskov, nickg
// Date:         1999-08-09
// Purpose:      JMR3904 platform IO support
// Description: 
// Usage:        #include <cyg/hal/plf_io.h>
//
//####DESCRIPTIONEND####
//
//=============================================================================

#include <pkgconf/hal.h>

//-----------------------------------------------------------------------------

#define	IRQ_TMR0	7
#define	IRQ_TMR1	6
#define	IRQ_UR		5
#define	IRQ_TMR2	4
#define	IRQ_MAC		3

#define MI_BASE     CYGNUM_HAL_REG_BASE
#define	TM_BASE		(MI_BASE+0x1000)
#define UR_BASE		(MI_BASE+0x2000)
#define	MAC_BASE	(MI_BASE+0x3000)
#define	IC_BASE		(MI_BASE+0x4000)
#define	GPIO_BASE	(MI_BASE+0x5000)

#define	MR1C		0x00
#define	MR2C		0x04
#define	MSDC		0x10
#define	MSR1C		0x14
#define	MSR1A		0x18
#define	MSR1D		0x1c
#define	MSR2C		0x20
#define	MSR2A		0x24
#define	MSR2D		0x28

#ifdef	OLD_TIMER
#define	T0CR		(0x00/4)	/* current value */
#define	T0LR		(0x04/4)	/* reload value */
#define	T1CR		(0x08/4)
#define	T1LR		(0x0c/4)
#define	T2CR		(0x10/4)
#define	T2LR		(0x14/4)
#define	TPR		(0x18/4)	/* prescaler */
#define	TCN		(0x1c/4)	/* control */
#define	TIC		(0x20/4)	/* interrupt clear */
#else
#define	T0CR		(0x00/4)	/* current value */
#define	T0LR		(0x04/4)	/* reload value */
#define	T0PR		(0x08/4)	/* prescaler */
#define	T0CN		(0x0c/4)	/* control */
#define T0IS		(0x10/4)	/* interrupt status */
#define	T1CR		(0x14/4)	/* current value */
#define	T1LR		(0x18/4)	/* reload value */
#define	T1PR		(0x1c/4)	/* prescaler */
#define	T1CN		(0x20/4)	/* control */
#define T1IS		(0x24/4)	/* interrupt status */
#define	T2CR		(0x28/4)	/* current value */
#define	T2LR		(0x2c/4)	/* reload value */
#define	T2PR		(0x30/4)	/* prescaler */
#define	T2CN		(0x34/4)	/* control */
#define T2IS		(0x38/4)	/* interrupt status */
#endif


#define	ISTS		(000>>2)
#define	IMSK		(0x4>>2)
#define	ISR1		(0x8>>2)
#define	ISR2		(0xc>>2)

#define	URBR		0x0			/* tx/rx buffer register */
#define	URCS		0x4			/* control/status */

#define	URCS_TIE	(1<<15)
#define	URCS_RIE	(1<<14)		
#define	URCS_PE		(1<<13)		
#define	URCS_EVEN	(1<<12)
#define	URCS_SP2	(1<<11)
#define	URCS_LB		(1<<10)
#define	URCS_P		(1<<9)
#define	URCS_PER	(1<<8)
#define	URCS_FE		(1<<7)
/* b4~b6 : rx threshold */ 
#define	URCS_TF		(1<<3)		/* tx fifo full */
#define	URCS_TE		(1<<2)		/* tx empty */
#define	URCS_RF		(1<<1)		/* rx full */
#define	URCS_TB		(1<<0)		/* tx busy */

#define	GPVAL 		(0>>2)
#define	GPSET 		(4>>2)
#define GPCLR 		(8>>2)
#define	GPDIR 		(0xc>>2)
#define GPSEL 		(0x10>>2)

#ifndef __ASSEMBLER__
#define	HAL_CAMELOT_SYSCLK \
  ((2!=((*(unsigned int *) (GPIO_BASE+0x18))&3))? /*!150Mhz*/	\
	125000000 : CYGNUM_HAL_RTC_CLOCK)
#define	HAL_CAMELOT_TMR_PERIOD \
  ((2!=((*(unsigned int *) (GPIO_BASE+0x18))&3))? /*!150Mhz*/	\
	(125000000/CYGNUM_HAL_RTC_PRESCALE/CYGNUM_HAL_RTC_DENOMINATOR ) : \
	(CYGNUM_HAL_RTC_CLOCK/CYGNUM_HAL_RTC_PRESCALE/CYGNUM_HAL_RTC_DENOMINATOR) ) 
#endif
//-----------------------------------------------------------------------------
// end of plf_io.h
#endif // CYGONCE_PLF_IO_H
